<TMPL_INCLUDE NAME="mail_header.tpl">

<p>
<span trspan="hello">Hello</span> <TMPL_VAR NAME="session_cn" ESCAPE=HTML>,<br />
<br />
<span trspan="resetCertificateOK">Your certificate has been successfully reset!</span> 
</p>

<TMPL_INCLUDE NAME="mail_footer.tpl">
