            </div>
            <div class="footer">
                <p>
                    <span trspan="autoMail">This mail was sent automatically</span><br />
                    <span trspan="requestIssuedFromIP">The request was issued from IP</span> <TMPL_VAR NAME="session_ipAddr" ESCAPE=HTML>
                </p>
		</div>
	</body>
</html>
