# This file is generated by Lemonldap::NG::Manager::Build. Don't modify it by hand
package Lemonldap::NG::Portal::Main::Constants;

use strict;
use Exporter 'import';

our $VERSION = '2.19.0';

use constant HANDLER => 'Lemonldap::NG::Handler::PSGI::Main';
use constant URIRE =>
qr{(((?^:https?))://((?:(?:(?:(?:(?:[a-zA-Z0-9][-a-zA-Z0-9]*)?[a-zA-Z0-9])[.])*(?:[a-zA-Z][-a-zA-Z0-9]*[a-zA-Z0-9]|[a-zA-Z])[.]?)|(?:[0-9]+[.][0-9]+[.][0-9]+[.][0-9]+)))(?::((?:[0-9]*)))?(/(((?:(?:(?:(?:[a-zA-Z0-9\-_.!~*'():\@&=+\$,]+|(?:%[a-fA-F0-9][a-fA-F0-9]))*)(?:;(?:(?:[a-zA-Z0-9\-_.!~*'():\@&=+\$,]+|(?:%[a-fA-F0-9][a-fA-F0-9]))*))*)(?:/(?:(?:(?:[a-zA-Z0-9\-_.!~*'():\@&=+\$,]+|(?:%[a-fA-F0-9][a-fA-F0-9]))*)(?:;(?:(?:[a-zA-Z0-9\-_.!~*'():\@&=+\$,]+|(?:%[a-fA-F0-9][a-fA-F0-9]))*))*))*))(?:[?]((?:(?:[;/?:\@&=+\$,a-zA-Z0-9\-_.!~*'()]+|(?:%[a-fA-F0-9][a-fA-F0-9]))*)))?))?)};
use constant {
    PE_2FWAIT                            => -6,
    PE_IDPCHOICE                         => -5,
    PE_SENDRESPONSE                      => -4,
    PE_INFO                              => -3,
    PE_REDIRECT                          => -2,
    PE_DONE                              => -1,
    PE_OK                                => 0,
    PE_SESSIONEXPIRED                    => 1,
    PE_FORMEMPTY                         => 2,
    PE_WRONGMANAGERACCOUNT               => 3,
    PE_USERNOTFOUND                      => 4,
    PE_BADCREDENTIALS                    => 5,
    PE_LDAPCONNECTFAILED                 => 6,
    PE_LDAPERROR                         => 7,
    PE_APACHESESSIONERROR                => 8,
    PE_FIRSTACCESS                       => 9,
    PE_BADCERTIFICATE                    => 10,
    PE_NO_PASSWORD_BE                    => 20,
    PE_PP_ACCOUNT_LOCKED                 => 21,
    PE_PP_PASSWORD_EXPIRED               => 22,
    PE_CERTIFICATEREQUIRED               => 23,
    PE_ERROR                             => 24,
    PE_PP_CHANGE_AFTER_RESET             => 25,
    PE_PP_PASSWORD_MOD_NOT_ALLOWED       => 26,
    PE_PP_MUST_SUPPLY_OLD_PASSWORD       => 27,
    PE_PP_INSUFFICIENT_PASSWORD_QUALITY  => 28,
    PE_PP_PASSWORD_TOO_SHORT             => 29,
    PE_PP_PASSWORD_TOO_YOUNG             => 30,
    PE_PP_PASSWORD_IN_HISTORY            => 31,
    PE_PP_GRACE                          => 32,
    PE_PP_EXP_WARNING                    => 33,
    PE_PASSWORD_MISMATCH                 => 34,
    PE_PASSWORD_OK                       => 35,
    PE_NOTIFICATION                      => 36,
    PE_BADURL                            => 37,
    PE_NOSCHEME                          => 38,
    PE_BADOLDPASSWORD                    => 39,
    PE_MALFORMEDUSER                     => 40,
    PE_SESSIONNOTGRANTED                 => 41,
    PE_CONFIRM                           => 42,
    PE_MAILFORMEMPTY                     => 43,
    PE_BADMAILTOKEN                      => 44,
    PE_MAILERROR                         => 45,
    PE_MAILOK                            => 46,
    PE_LOGOUT_OK                         => 47,
    PE_SAML_ERROR                        => 48,
    PE_SAML_LOAD_SERVICE_ERROR           => 49,
    PE_SAML_LOAD_IDP_ERROR               => 50,
    PE_SAML_SSO_ERROR                    => 51,
    PE_SAML_UNKNOWN_ENTITY               => 52,
    PE_SAML_DESTINATION_ERROR            => 53,
    PE_SAML_CONDITIONS_ERROR             => 54,
    PE_SAML_IDPSSOINITIATED_NOTALLOWED   => 55,
    PE_SLO_ERROR                         => 56,
    PE_SAML_SIGNATURE_ERROR              => 57,
    PE_SAML_ART_ERROR                    => 58,
    PE_SAML_SESSION_ERROR                => 59,
    PE_SAML_LOAD_SP_ERROR                => 60,
    PE_SAML_ATTR_ERROR                   => 61,
    PE_OPENID_EMPTY                      => 62,
    PE_OPENID_BADID                      => 63,
    PE_MISSINGREQATTR                    => 64,
    PE_BADPARTNER                        => 65,
    PE_MAILCONFIRMATION_ALREADY_SENT     => 66,
    PE_PASSWORDFORMEMPTY                 => 67,
    PE_CAS_SERVICE_NOT_ALLOWED           => 68,
    PE_MAILFIRSTACCESS                   => 69,
    PE_MAILNOTFOUND                      => 70,
    PE_PASSWORDFIRSTACCESS               => 71,
    PE_MAILCONFIRMOK                     => 72,
    PE_RADIUSCONNECTFAILED               => 73,
    PE_MUST_SUPPLY_OLD_PASSWORD          => 74,
    PE_FORBIDDENIP                       => 75,
    PE_CAPTCHAERROR                      => 76,
    PE_CAPTCHAEMPTY                      => 77,
    PE_REGISTERFIRSTACCESS               => 78,
    PE_REGISTERFORMEMPTY                 => 79,
    PE_REGISTERALREADYEXISTS             => 80,
    PE_NOTOKEN                           => 81,
    PE_TOKENEXPIRED                      => 82,
    PE_U2FFAILED                         => 83,
    PE_UNAUTHORIZEDPARTNER               => 84,
    PE_RENEWSESSION                      => 85,
    PE_WAIT                              => 86,
    PE_MUSTAUTHN                         => 87,
    PE_MUSTHAVEMAIL                      => 88,
    PE_SAML_SERVICE_NOT_ALLOWED          => 89,
    PE_OIDC_SERVICE_NOT_ALLOWED          => 90,
    PE_OID_SERVICE_NOT_ALLOWED           => 91,
    PE_GET_SERVICE_NOT_ALLOWED           => 92,
    PE_IMPERSONATION_SERVICE_NOT_ALLOWED => 93,
    PE_ISSUERMISSINGREQATTR              => 94,
    PE_DECRYPTVALUE_SERVICE_NOT_ALLOWED  => 95,
    PE_BADOTP                            => 96,
    PE_RESETCERTIFICATE_INVALID          => 97,
    PE_RESETCERTIFICATE_FORMEMPTY        => 98,
    PE_RESETCERTIFICATE_FIRSTACCESS      => 99,
    PE_PP_NOT_ALLOWED_CHARACTER          => 100,
    PE_PP_NOT_ALLOWED_CHARACTERS         => 101,
    PE_UPGRADESESSION                    => 102,
    PE_NO_SECOND_FACTORS                 => 103,
    PE_BAD_DEVOPS_FILE                   => 104,
    PE_FILENOTFOUND                      => 105,
    PE_OIDC_AUTH_ERROR                   => 106,
    PE_UNKNOWNPARTNER                    => 107,
    PE_UNAUTHORIZEDURL                   => 108,
    PE_UNPROTECTEDURL                    => 109,
    PE_RETRY_2FA                         => 110,
};

sub portalConsts {
    return {
        '-1'  => 'PE_DONE',
        '-2'  => 'PE_REDIRECT',
        '-3'  => 'PE_INFO',
        '-4'  => 'PE_SENDRESPONSE',
        '-5'  => 'PE_IDPCHOICE',
        '-6'  => 'PE_2FWAIT',
        '0'   => 'PE_OK',
        '1'   => 'PE_SESSIONEXPIRED',
        '10'  => 'PE_BADCERTIFICATE',
        '100' => 'PE_PP_NOT_ALLOWED_CHARACTER',
        '101' => 'PE_PP_NOT_ALLOWED_CHARACTERS',
        '102' => 'PE_UPGRADESESSION',
        '103' => 'PE_NO_SECOND_FACTORS',
        '104' => 'PE_BAD_DEVOPS_FILE',
        '105' => 'PE_FILENOTFOUND',
        '106' => 'PE_OIDC_AUTH_ERROR',
        '107' => 'PE_UNKNOWNPARTNER',
        '108' => 'PE_UNAUTHORIZEDURL',
        '109' => 'PE_UNPROTECTEDURL',
        '110' => 'PE_RETRY_2FA',
        '2'   => 'PE_FORMEMPTY',
        '20'  => 'PE_NO_PASSWORD_BE',
        '21'  => 'PE_PP_ACCOUNT_LOCKED',
        '22'  => 'PE_PP_PASSWORD_EXPIRED',
        '23'  => 'PE_CERTIFICATEREQUIRED',
        '24'  => 'PE_ERROR',
        '25'  => 'PE_PP_CHANGE_AFTER_RESET',
        '26'  => 'PE_PP_PASSWORD_MOD_NOT_ALLOWED',
        '27'  => 'PE_PP_MUST_SUPPLY_OLD_PASSWORD',
        '28'  => 'PE_PP_INSUFFICIENT_PASSWORD_QUALITY',
        '29'  => 'PE_PP_PASSWORD_TOO_SHORT',
        '3'   => 'PE_WRONGMANAGERACCOUNT',
        '30'  => 'PE_PP_PASSWORD_TOO_YOUNG',
        '31'  => 'PE_PP_PASSWORD_IN_HISTORY',
        '32'  => 'PE_PP_GRACE',
        '33'  => 'PE_PP_EXP_WARNING',
        '34'  => 'PE_PASSWORD_MISMATCH',
        '35'  => 'PE_PASSWORD_OK',
        '36'  => 'PE_NOTIFICATION',
        '37'  => 'PE_BADURL',
        '38'  => 'PE_NOSCHEME',
        '39'  => 'PE_BADOLDPASSWORD',
        '4'   => 'PE_USERNOTFOUND',
        '40'  => 'PE_MALFORMEDUSER',
        '41'  => 'PE_SESSIONNOTGRANTED',
        '42'  => 'PE_CONFIRM',
        '43'  => 'PE_MAILFORMEMPTY',
        '44'  => 'PE_BADMAILTOKEN',
        '45'  => 'PE_MAILERROR',
        '46'  => 'PE_MAILOK',
        '47'  => 'PE_LOGOUT_OK',
        '48'  => 'PE_SAML_ERROR',
        '49'  => 'PE_SAML_LOAD_SERVICE_ERROR',
        '5'   => 'PE_BADCREDENTIALS',
        '50'  => 'PE_SAML_LOAD_IDP_ERROR',
        '51'  => 'PE_SAML_SSO_ERROR',
        '52'  => 'PE_SAML_UNKNOWN_ENTITY',
        '53'  => 'PE_SAML_DESTINATION_ERROR',
        '54'  => 'PE_SAML_CONDITIONS_ERROR',
        '55'  => 'PE_SAML_IDPSSOINITIATED_NOTALLOWED',
        '56'  => 'PE_SLO_ERROR',
        '57'  => 'PE_SAML_SIGNATURE_ERROR',
        '58'  => 'PE_SAML_ART_ERROR',
        '59'  => 'PE_SAML_SESSION_ERROR',
        '6'   => 'PE_LDAPCONNECTFAILED',
        '60'  => 'PE_SAML_LOAD_SP_ERROR',
        '61'  => 'PE_SAML_ATTR_ERROR',
        '62'  => 'PE_OPENID_EMPTY',
        '63'  => 'PE_OPENID_BADID',
        '64'  => 'PE_MISSINGREQATTR',
        '65'  => 'PE_BADPARTNER',
        '66'  => 'PE_MAILCONFIRMATION_ALREADY_SENT',
        '67'  => 'PE_PASSWORDFORMEMPTY',
        '68'  => 'PE_CAS_SERVICE_NOT_ALLOWED',
        '69'  => 'PE_MAILFIRSTACCESS',
        '7'   => 'PE_LDAPERROR',
        '70'  => 'PE_MAILNOTFOUND',
        '71'  => 'PE_PASSWORDFIRSTACCESS',
        '72'  => 'PE_MAILCONFIRMOK',
        '73'  => 'PE_RADIUSCONNECTFAILED',
        '74'  => 'PE_MUST_SUPPLY_OLD_PASSWORD',
        '75'  => 'PE_FORBIDDENIP',
        '76'  => 'PE_CAPTCHAERROR',
        '77'  => 'PE_CAPTCHAEMPTY',
        '78'  => 'PE_REGISTERFIRSTACCESS',
        '79'  => 'PE_REGISTERFORMEMPTY',
        '8'   => 'PE_APACHESESSIONERROR',
        '80'  => 'PE_REGISTERALREADYEXISTS',
        '81'  => 'PE_NOTOKEN',
        '82'  => 'PE_TOKENEXPIRED',
        '83'  => 'PE_U2FFAILED',
        '84'  => 'PE_UNAUTHORIZEDPARTNER',
        '85'  => 'PE_RENEWSESSION',
        '86'  => 'PE_WAIT',
        '87'  => 'PE_MUSTAUTHN',
        '88'  => 'PE_MUSTHAVEMAIL',
        '89'  => 'PE_SAML_SERVICE_NOT_ALLOWED',
        '9'   => 'PE_FIRSTACCESS',
        '90'  => 'PE_OIDC_SERVICE_NOT_ALLOWED',
        '91'  => 'PE_OID_SERVICE_NOT_ALLOWED',
        '92'  => 'PE_GET_SERVICE_NOT_ALLOWED',
        '93'  => 'PE_IMPERSONATION_SERVICE_NOT_ALLOWED',
        '94'  => 'PE_ISSUERMISSINGREQATTR',
        '95'  => 'PE_DECRYPTVALUE_SERVICE_NOT_ALLOWED',
        '96'  => 'PE_BADOTP',
        '97'  => 'PE_RESETCERTIFICATE_INVALID',
        '98'  => 'PE_RESETCERTIFICATE_FORMEMPTY',
        '99'  => 'PE_RESETCERTIFICATE_FIRSTACCESS'
    };

}

# EXPORTER PARAMETERS
our @EXPORT_OK = (
    'portalConsts',
    'HANDLER',
    'URIRE',
    'PE_2FWAIT',
    'PE_IDPCHOICE',
    'PE_SENDRESPONSE',
    'PE_INFO',
    'PE_REDIRECT',
    'PE_DONE',
    'PE_OK',
    'PE_SESSIONEXPIRED',
    'PE_FORMEMPTY',
    'PE_WRONGMANAGERACCOUNT',
    'PE_USERNOTFOUND',
    'PE_BADCREDENTIALS',
    'PE_LDAPCONNECTFAILED',
    'PE_LDAPERROR',
    'PE_APACHESESSIONERROR',
    'PE_FIRSTACCESS',
    'PE_BADCERTIFICATE',
    'PE_NO_PASSWORD_BE',
    'PE_PP_ACCOUNT_LOCKED',
    'PE_PP_PASSWORD_EXPIRED',
    'PE_CERTIFICATEREQUIRED',
    'PE_ERROR',
    'PE_PP_CHANGE_AFTER_RESET',
    'PE_PP_PASSWORD_MOD_NOT_ALLOWED',
    'PE_PP_MUST_SUPPLY_OLD_PASSWORD',
    'PE_PP_INSUFFICIENT_PASSWORD_QUALITY',
    'PE_PP_PASSWORD_TOO_SHORT',
    'PE_PP_PASSWORD_TOO_YOUNG',
    'PE_PP_PASSWORD_IN_HISTORY',
    'PE_PP_GRACE',
    'PE_PP_EXP_WARNING',
    'PE_PASSWORD_MISMATCH',
    'PE_PASSWORD_OK',
    'PE_NOTIFICATION',
    'PE_BADURL',
    'PE_NOSCHEME',
    'PE_BADOLDPASSWORD',
    'PE_MALFORMEDUSER',
    'PE_SESSIONNOTGRANTED',
    'PE_CONFIRM',
    'PE_MAILFORMEMPTY',
    'PE_BADMAILTOKEN',
    'PE_MAILERROR',
    'PE_MAILOK',
    'PE_LOGOUT_OK',
    'PE_SAML_ERROR',
    'PE_SAML_LOAD_SERVICE_ERROR',
    'PE_SAML_LOAD_IDP_ERROR',
    'PE_SAML_SSO_ERROR',
    'PE_SAML_UNKNOWN_ENTITY',
    'PE_SAML_DESTINATION_ERROR',
    'PE_SAML_CONDITIONS_ERROR',
    'PE_SAML_IDPSSOINITIATED_NOTALLOWED',
    'PE_SLO_ERROR',
    'PE_SAML_SIGNATURE_ERROR',
    'PE_SAML_ART_ERROR',
    'PE_SAML_SESSION_ERROR',
    'PE_SAML_LOAD_SP_ERROR',
    'PE_SAML_ATTR_ERROR',
    'PE_OPENID_EMPTY',
    'PE_OPENID_BADID',
    'PE_MISSINGREQATTR',
    'PE_BADPARTNER',
    'PE_MAILCONFIRMATION_ALREADY_SENT',
    'PE_PASSWORDFORMEMPTY',
    'PE_CAS_SERVICE_NOT_ALLOWED',
    'PE_MAILFIRSTACCESS',
    'PE_MAILNOTFOUND',
    'PE_PASSWORDFIRSTACCESS',
    'PE_MAILCONFIRMOK',
    'PE_RADIUSCONNECTFAILED',
    'PE_MUST_SUPPLY_OLD_PASSWORD',
    'PE_FORBIDDENIP',
    'PE_CAPTCHAERROR',
    'PE_CAPTCHAEMPTY',
    'PE_REGISTERFIRSTACCESS',
    'PE_REGISTERFORMEMPTY',
    'PE_REGISTERALREADYEXISTS',
    'PE_NOTOKEN',
    'PE_TOKENEXPIRED',
    'PE_U2FFAILED',
    'PE_UNAUTHORIZEDPARTNER',
    'PE_RENEWSESSION',
    'PE_WAIT',
    'PE_MUSTAUTHN',
    'PE_MUSTHAVEMAIL',
    'PE_SAML_SERVICE_NOT_ALLOWED',
    'PE_OIDC_SERVICE_NOT_ALLOWED',
    'PE_OID_SERVICE_NOT_ALLOWED',
    'PE_GET_SERVICE_NOT_ALLOWED',
    'PE_IMPERSONATION_SERVICE_NOT_ALLOWED',
    'PE_ISSUERMISSINGREQATTR',
    'PE_DECRYPTVALUE_SERVICE_NOT_ALLOWED',
    'PE_BADOTP',
    'PE_RESETCERTIFICATE_INVALID',
    'PE_RESETCERTIFICATE_FORMEMPTY',
    'PE_RESETCERTIFICATE_FIRSTACCESS',
    'PE_PP_NOT_ALLOWED_CHARACTER',
    'PE_PP_NOT_ALLOWED_CHARACTERS',
    'PE_UPGRADESESSION',
    'PE_NO_SECOND_FACTORS',
    'PE_BAD_DEVOPS_FILE',
    'PE_FILENOTFOUND',
    'PE_OIDC_AUTH_ERROR',
    'PE_UNKNOWNPARTNER',
    'PE_UNAUTHORIZEDURL',
    'PE_UNPROTECTEDURL',
    'PE_RETRY_2FA'
);
our %EXPORT_TAGS = ( 'all' => [ @EXPORT_OK, 'import' ], );

our @EXPORT = qw(import PE_OK);

1;
