<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
	<head>
		<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
		<style type="text/css">
		body {margin: 0; padding: 0; min-width: 100%!important; background: #eee; font-family: sans-serif;}
		.content {width: 100%; max-width: 600px; background: #fff; color: #000; font-weight: 500; padding: 10px; margin: 20px auto; border-radius: 30px;}
        .footer  {width: 100%; max-width: 600px; color:#454545; font-weight:300; margin: 0 10px;}
		</style>
	</head>
	<body>
        <div class="content">
